﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="JavaScriptModuleInclude.cs" company="Devbridge Group LLC">
// 
// Copyright (C) 2015,2016 Devbridge Group LLC
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/. 
// </copyright>
// 
// <summary>
// Better CMS is a publishing focused and developer friendly .NET open source CMS.
// 
// Website: https://www.bettercms.com 
// GitHub: https://github.com/devbridge/bettercms
// Email: info@bettercms.com
// </summary>
// --------------------------------------------------------------------------------------------------------------------
namespace BetterCms.Module.Root.ViewModels.Rendering
{
    /// <summary>
    /// JS module view model.
    /// </summary>
    public class JavaScriptModuleInclude
    {
        /// <summary>
        /// Gets or sets a value indicating whether this JavaScript module is auto generated.
        /// </summary>
        /// <value>
        /// <c>true</c> if this JavaScript module is auto generated; otherwise, <c>false</c>.
        /// </value>
        public bool IsAutoGenerated { get; set; }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>
        /// The name.
        /// </value>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the 'friendly' module name to use internally.
        /// </summary>
        /// <value>
        /// The 'friendly' module name to use internally.
        /// </value>
        public string FriendlyName { get; set; }

        /// <summary>
        /// Gets or sets the path.
        /// </summary>
        /// <value>
        /// The path.
        /// </value>
        public string Path { get; set; }

        /// <summary>
        /// Gets or sets the minified path.
        /// </summary>
        /// <value>
        /// The minified path.
        /// </value>
        public string MinifiedPath { get; set; }

        /// <summary>
        /// Gets or sets registered links.
        /// </summary>
        /// <value>
        /// A list of registered links.
        /// </value>
        public ProjectionsViewModel Links { get; set; }

        /// <summary>
        /// Gets or sets the globalization.
        /// </summary>
        /// <value>
        /// The globalization.
        /// </value>
        public ProjectionsViewModel Globalization { get; set; }

        /// <summary>
        /// Gets or sets the shim configuration.
        /// </summary>
        /// <value>
        /// The shim configuration.
        /// </value>
        public JavaScriptModuleShimConfigurationViewModel ShimConfig { get; set; }

        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("Name: {0}, FriendlyName: {1}, Path: {2}", Name, FriendlyName, Path);
        }
    }
}